#!/usr/bin/env python3
"""
Script to create a new issue on an existing project
in GitLab.
You need to provide an valid TOKEN to create new issues

You also need to provide the ID of the project,
where you want to add the issue, e.g.:
  python3 create_issue.py 123

The issue data itself is simple hard coded in this example
script and would typically come from CLI, a file etc.
"""

import urllib.request
import json
import sys
import os

def read_token_from_file(fn="api_token.txt"):
    if not os.path.isfile(fn):
        return ""
    lines=list(filter(lambda l: l.startswith("API_TOKEN="), open("api_token.txt").readlines() ))
    if len(lines) <1:
        return ""
    return lines.pop().strip().split("API_TOKEN=").pop()
    

TOKEN = read_token_from_file()

if len(sys.argv) != 2:
    sys.exit(f"invalid value(s)\n  usage: python3 {sys.argv[0]} <ID>")

id = sys.argv[1]

# https://docs.gitlab.com/ee/api/issues.html#new-issue
api_endpoint = f'https://mode.fh-joanneum.at/api/v4/projects/{id}/issues'

# https://docs.gitlab.com/ee/api/rest/#authentication
headers = { 
    'Authorization': f'Bearer {TOKEN}', 
    'Content-Type': 'application/json'    
}

data = {
    'title': 'this is a test by jf',
    'labels': 'example,test',
    'description': 'this is an example to demonstrate the API'
}
json_data = json.dumps(data).encode('utf8')

request = urllib.request.Request(api_endpoint, data=json_data, headers=headers, method='POST')

try:
    with urllib.request.urlopen(request) as response:
        response_data = response.read().decode('utf8')
        print("*****")
        print("Successfully created the issue. Response from server: ")
        print(json.dumps(json.loads(response_data), indent=2))
        print("*****")
except urllib.error.HTTPError as e:
    print(f"HTTP Error {e.code}: {e.reason}", file=sys.stderr)
except urllib.error.URLError as e:
    print(f"URL Error: {e.reason}", file=sys.stderr)
except Exception as e:
    print(f"unexpected Error: {e}", file=sys.stderr)
