#!/usr/bin/env python3
import re
import sys
from pathlib import Path

print("Input validation using regular expressions (regex)")

if len(sys.argv) < 2:
    sys.exit(f"Usage: {sys.argv[0]} <filename>")

filename = Path(sys.argv[1])

# We accept data files with suffix: .txt .text .TXT .TEXT .tXt ...
suffix_match = re.search(r"\.te?xt", filename.name, re.IGNORECASE)
if not suffix_match:
    sys.exit(f"\tERR: '{filename}' not accepted. Please provide a *.txt file!")

if not filename.is_file():
    sys.exit(f"provided file '{filename}' does not exist!")


# We extract LEFT, OPERATOR, RIGHT from strings such as
#    e.g.: "13 * 54"
#    e.g.: "3+ 54"
#    e.g.: "13 -4"
# Note: only integer numbers and operators +, -, * are supported.
def extract_with_regex(user_input):
    calc_match = re.match(  # Very strict pattern matching with regular expressions
        r"""            # For patterns we use Python "raw" strings with r""
        ^               # From the beginning of the line
            (           # first match group = LEFT number
             \d+        #   one or more digits (i.e. an arbitrary number)
            )
        \s*             # zero or more whitespaces (ie. optional)
            (           # second match group = math OPERATOR
             [\*\-\+]   #   one of the given characters '*' or '-' or '+' (i.e. operators)
                        #   Note: we escaped special chars like * with \*
            )
        \s*             # zero or more whitespaces (ie. optional)
            (           # third match group = RIGHT number
             \d+        #   one or more digits (i.e. an arbitrary number)
            )
        $               # To end of line
        """,
        user_input,  # e.g.: "0 *33"
        re.VERBOSE,  # all whitespaces and comments are ignored
    )
    # short version of the regular expression above:
    # calc_match = re.match(r"^(\d+\s*)([\*-+])\s*(\d+)$", user_input)

    # print(f"\t DBG: matching '{user_input}' => '{ok}'.")
    if not calc_match:
        raise Exception("Unallowed input. Int calculations + - * are supported")
    (left, op, right) = calc_match.groups()
    # print(f"\tDBG: we will user operator '{op}' to process '{left}' and '{right}'.")
    return int(left), op, int(right)


input_lines = open(filename).readlines()

sum = 0
for idx, line in enumerate(input_lines, start=1):
    line = line.strip()
    print(f"{idx}/{len(input_lines)}: we add '{line}'...")
    try:  # parsing might fail on erroneous input
        left, opr, right = extract_with_regex(line)
    except Exception as err:
        print(f"\tSorry, we are unsure about input '{line}'. We better ignore this.")
        print(f"\t      Error: '{err}'.")
        continue
    print(f"\t=> DBG: we performed {sum} + {left} {opr} {right}", end=" => ")
    # Security enhanced:
    #   Whitelisting by explicit checking for allowed operation
    #   => no 'eval' or 'exec' method used and more!!
    if opr == "+":
        sum += left + right
    elif opr == "-":
        sum += left - right
    elif opr == "*":
        sum += left * right
    else:
        print("WARN: Unexpected operator '{opr}'. Not implemented.")
    print(f"SUM = {sum}")
print(f"Final result: Sum = {sum}.")
